<?php
// phpcs:disable WordPress.PHP.DevelopmentFunctions
// phpcs:disable WordPress.Files.FileName.InvalidClassFileName
// phpcs:disable WordPress.WP.AlternativeFunctions.json_encode_json_encode
// phpcs:disable WordPress.Files.FileName.NotHyphenatedLowercase
// phpcs:disable WordPress.NamingConventions.ValidVariableName.PropertyNotSnakeCase

namespace WeeConnectPay\WordPress\Plugin\includes;

use WC_Order;
use WC_Order_Item;
use WeeConnectPay\Exceptions\Codes\ExceptionCode;
use WeeConnectPay\Exceptions\MissingDependencyException;
use WeeConnectPay\Exceptions\MissingStateException;
use WeeConnectPay\Exceptions\WeeConnectPayException;
use WeeConnectPay\Integrations\Authentication;
use WeeConnectPay\Integrations\IntegrationSettings;
use WeeConnectPay\Integrations\PaymentFields;
use WeeConnectPay\Settings;
use WeeConnectPay\StandardizedResponse;
use WP_Error;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 *  Class responsible for external calls to our API.
 *
 * @package WeeConnectPay\WordPress\Plugin\includes
 */
class WeeConnectPayAPI {

	/**
	 * Allows for quick toggling of debug logging on any environment. Set to false for production.
	 * @var bool
	 */
	private $is_debug = true;
	/**
	 * Determines if we should connect to the sandbox. Set to false for production.
	 * @var bool
	 */
	private $is_sandbox = true;
	/**
	 * Determines if we should connect to a locally hosted dev environment of our API.
	 * @var bool
	 */
	private $is_dev_sandbox = true;
	/**
	 * Determines if we should connect with a publicly exposed (ngrok) local WP install to the staging API server.
	 * @var bool
	 */
	public $is_local_sandbox_to_staging;
	/**
	 * Publicly exposed (ngrok) WordPress URL.
	 * @var string
	 */
	public $local_sandbox_ngrok_url;
	/**
	 * JSON Web Token - For authenticating calls to protected API endpoints.
	 * @var string
	 */
	private $jwt;
	/**
	 * URL of our API.
	 * @var string
	 */
	public $url_api;
	/**
	 * Settings array for our plugin.
	 * @var array
	 */
	public $settings;

	public $wp_env;

	/**
	 * Integration Settings Class.
	 * @var IntegrationSettings
	 */
	public $integrationSettings;

	const LOCAL_API_URL = "https://weeconnect-api.test";
	const DEV_API_URL = "";
	const STAGING_API_URL = "https://apidev.weeconnectpay.com";
	const PRODUCTION_API_URL = "https://api.weeconnectpay.com";


	/**
	 * @updated 1.4.1
	 */
	public function __construct() {
		$this->url_api = self::getApiDomain();
		$this->integrationSettings = new IntegrationSettings();
		$this->wp_env = WeeConnectPayUtilities::get_wp_env();
	}

//	public function get_checkout_url( $payment_processor ) {
//		return $this->url_api . "/v1/$payment_processor[pp_type]/order/$payment_processor[uuid]/checkout";
//	}




	/**
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public function prepare_clover_discounts( WC_Order $order ) {

		$discount        = $order->get_total_discount();
		$clover_discount = array();

		if ( $discount > 0 ) {

			$discount_name = 'Coupons : - ';
			foreach ( $order->get_coupons() as $coupon_name ) {
				$discount_name .= wc_clean( $coupon_name ) . '-';
			}

			$clover_discount = array(
				'value' => $discount,
				'type'  => 'amount',
				'name'  => $discount_name,
			);
		}

		return $clover_discount;
	}

	/**
	 * @param WC_Order $order
	 * @param string $payment_processing_source
	 * @param string $payment_processing_source_type
	 *
	 * @return array
	 */
	public function prepare_internal_woocommerce_order( WC_Order $order, string $payment_processing_source, string $payment_processing_source_type ): array {
		return array(
			'woocommerce_order_id'           => $order->get_id(),
			'amount'                         => round( $order->get_total() * 100.00 ), // In cents
			'thank_you_url'                  => $this->get_woocommerce_thank_you_url( $order ),
			'callback_url'                   => $this->get_woocommerce_callback_url(),
			'payment_processing_source'      => $payment_processing_source,
			'payment_processing_source_type' => $payment_processing_source_type,
			'status'                         => $order->get_status(),
		);
	}


	public function get_clover_order_total_from_products( $clover_order ) {
		$total = 0;
		foreach ( $clover_order['items'] as $item ) {
			$is_clover_imported_product = isset( $item['id'] );
			if ( $is_clover_imported_product ) {
				// multiply amount by quantity, add taxes if any
				$item_subtotal = $item['amount'] * $item['quantity'];

				$item_combined_tax_rates = 0;
				$item_taxes              = 0;
				if ( isset( $item['tax_rates'] ) ) {
					foreach ( $item['tax_rates'] as $index => $tax_rate ) {
						$item_combined_tax_rates += $tax_rate->rate; // Get each tax rate for this item
					}
					$formatted_combined_tax_rates = ( $item_combined_tax_rates / 10000000 );
					$item_taxes = $formatted_combined_tax_rates * $item_subtotal; // Process the combined tax rates to the item_subtotal
				} else {

				}

				$total += (int) round( $item_subtotal + $item_taxes ); // Add the combined taxes for the item total -- We only type cast here as a tax workaround, not in the actual calculation done by the back-end.
			} else {
				$total += $item['amount'];
			}
		}
		return $total;
	}


	public function format_clover_order_items( WC_Order $order ) {

		$items          = $this->get_items_as_clover_order_items( $order );
		$tax_items      = array();
		$shipping_items = $this->get_shipping_as_clover_items( $order );

		if ( count( $shipping_items ) > 0 ) {
			$items = array_merge( $items, $shipping_items );
		}

		return $items;
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public function get_taxes_as_clover_items( WC_Order $order ): array {
		$tax_items = array();

		foreach ( $order->get_taxes() as $tax ) {
			// Make tax item array
			array_push(
				$tax_items,
				array(
					'type'        => 'tax',
					'amount'      => round( ( (float) $tax->get_tax_total() + (float) $tax->get_shipping_tax_total() ) * 100 ),
					'description' => $tax->get_label(),
					'currency'    => strtolower( $order->get_currency() ),
					'quantity'    => 1,
				)
			);
		}

		return $tax_items;
	}

	/**
	 * @param WC_Order_Item $item
	 *
	 * @return string
	 */
	public function get_individual_item_tax( WC_Order_Item $item ) {
		return $item->get_total_tax();
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public function get_items_as_clover_order_items( WC_Order $order ): array {
		$items = array();

		$clover_inventory_items = array();
		foreach ( $order->get_items() as $item_id => $item ) {
			// Clover Inventory Import changes
			$clover_inventory_items[ $item_id ] = $this->set_clover_item_tax_rates_for_meta( $item );

			array_push( $items, $this->get_clover_formatted_item( $item, $order ) );
		}

		return $items;
	}

	public function set_clover_item_tax_rates_for_meta( WC_Order_Item $item ) {
		$item_tax_rates = array();

		// Check for import
		$product             = $item->get_product();
		$weeconnectpay_metadata = json_decode( json_encode( $product->get_meta( 'weeconnectpay_metadata' ) ) ); // Because PHP is a primitive language
		if ( isset( $weeconnectpay_metadata ) ) {
			if ( isset( $weeconnectpay_metadata->clover_imported ) ) {
				if ( isset( $weeconnectpay_metadata->clover->product_id, $weeconnectpay_metadata->clover->product_tax_rates ) ) {
					//die();
					$item_tax_rates = array();
				}
			}
		}

		return $item_tax_rates;
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public function get_shipping_as_clover_items( WC_Order $order ): array {
		$shipping_item  = array();
		$shipping_total = WeeConnectPayHelper::safe_amount_to_cents_int( $order->get_shipping_total() ) + WeeConnectPayHelper::safe_amount_to_cents_int( $order->get_shipping_tax() );

		if ( $shipping_total > 0 ) {
			array_push(
				$shipping_item,
				array(
					'amount'      => $shipping_total,
					// Only if we have shipping as a line item -- Edge case otherwise
					'description' => $order->get_meta( 'weeconnectpay_shipping_line_item_name' ),
					'currency'    => strtolower( $order->get_currency() ),
					'quantity'    => 1,
				)
			);
		}

		return $shipping_item;
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return string
	 */
	public function get_woocommerce_thank_you_url( WC_Order $order ) {
		return str_replace( 'http:', 'https:', $order->get_checkout_order_received_url() );
	}

	/**
	 * @return string|string[]
	 */
	public function get_woocommerce_callback_url() {
		$permalink = get_option('permalink_structure');
		$pattern = "/^\/index\.php/";
		$prefix = preg_match( $pattern, $permalink) ? '/index.php/' : '/';

		return str_replace( 'http:', 'https:', home_url( $prefix ) . 'wc-api/callback_wc_gateway_weeconnectpay' );
	}

	/**
	 * @param WC_Order_Item $item
	 * @param WC_Order $order
	 *
	 * @return array
	 */
	public function get_clover_formatted_item( \WC_Order_Item_Product $item, WC_Order $order ): array {

		$active_price = $item->get_total(); // The product active raw price after discount
		$amount       = WeeConnectPayHelper::safe_amount_to_cents_int( $active_price );

		if ( $item->get_tax_status() === 'taxable' ) {
			$amount += WeeConnectPayHelper::safe_amount_to_cents_int( $this->get_individual_item_tax( $item ) );
		}
		$formatted_amount = (int) $amount;

		$item_array = array(
			'type'        => 'sku',
			'amount'      => $formatted_amount,
			'description' => WeeConnectPayHelper::name_and_qty_as_clover_line_desc( $item->get_name(), $item->get_quantity() ),
			'currency'    => strtolower( $order->get_currency() ),
			'quantity'    => 1,
		);

		// Check for import
		$product             = $item->get_product();
		$weeconnectpay_metadata = json_decode( json_encode( $product->get_meta( 'weeconnectpay_metadata' ) ) ); // Because PHP is a primitive language
		if ( isset( $weeconnectpay_metadata ) && '' !== $weeconnectpay_metadata ) {
			if ( true === $weeconnectpay_metadata->clover_imported ) {
				if ( isset( $weeconnectpay_metadata->clover->product_id ) ) {

					$item_array = array(
						'type'        => 'sku',
						'amount'      => WeeConnectPayHelper::safe_amount_to_cents_int( $item->get_total() / $item->get_quantity() ),
						// The product active raw price after discount,
						'description' => $item->get_name(),
						'currency'    => strtolower( $order->get_currency() ),
						'quantity'    => $item->get_quantity(),
						'id'          => $weeconnectpay_metadata->clover->product_id,
						'tax_rates'   => $weeconnectpay_metadata->clover->product_tax_rates,
					);
				} else {
				}
			} else {
			}
		}

		return $item_array;
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return array[]
	 * @throws MissingStateException
	 *
	 * @updated 2.0.6
	 */
	public function prepare_clover_shipping_info( WC_Order $order ): array {

		// Fallback for missing "optional" county for GB
		if ( !$order->get_shipping_state() ) {
			throw new MissingStateException(
					__( 'A shipping address state, county or province is required for this gateway.', 'weeconnectpay' ),
				ExceptionCode::MISSING_SHIPPING_STATE
			);
		}

		return array(
			'shipping' => array(
				'address' => array(
					'city'        => $order->get_shipping_city(),
					'country'     => $order->get_shipping_country(),
					'line1'       => $order->get_shipping_address_1(),
					'line2'       => $order->get_shipping_address_2(),
					'postal_code' => $order->get_shipping_postcode(),
					'state'       => $order->get_shipping_state(),

				),
				'name'    => $order->get_formatted_shipping_full_name(),
				'phone'   => $order->get_billing_phone(),
			),
		);
	}


	public function orders_totals_matches( $woocommerce_order_total, $clover_order_total ) {
		// We don't want to cast anything because of binary representation is imprecise.
		return $woocommerce_order_total === $clover_order_total;
	}

	/**
	 * @param array $clover_order
	 * @param int $amount_in_cents
	 *
	 * @return bool
	 */
	public function is_matching_amounts( array $clover_order, int $amount_in_cents ): bool {
		$clover_total = $this->get_clover_order_total_from_products( $clover_order );
		if ( $this->orders_totals_matches( $amount_in_cents, $clover_total ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Prepare the order payload, we also use this order to validate amounts beforehand
	 *
	 * @param WC_Order $order
	 *
	 * @return array
	 * @throws MissingStateException
	 */
	public function prepare_clover_order( WC_Order $order ): array {
		$clover_discounts   = $this->prepare_clover_discounts( $order );
		$clover_order_items = $this->format_clover_order_items( $order );

		$clover_order = array(
			'items'    => $clover_order_items,
			'currency' => strtolower( $order->get_currency() ),
			'email'    => $order->get_billing_email() ?? null,
		);
		if ( $order->has_shipping_address() ) {
			$shipping_info = $this->prepare_clover_shipping_info( $order );
			$clover_order  = array_merge( $clover_order, $shipping_info );
		}

		return $clover_order;
	}

	// Since we want to use the settings class as a single source of truth, we should access it directly and not as a local instance of itself.
//	/**
//	 * @param IntegrationSettings $integrationSettings
//	 *
//	 * @return WeeConnectPayAPI
//	 */
//	public function setIntegrationSettings( IntegrationSettings $integrationSettings ): WeeConnectPayAPI {
//		$this->integrationSettings = $integrationSettings;
//
//		return $this;
//	}
//
//	/**
//	 * @return IntegrationSettings
//	 */
//	public function getIntegrationSettings(): IntegrationSettings {
//		return $this->integrationSettings;
//	}

	private function api_get( $url, $args = array() ) {
		$args = $this->set_request_args( $args );

		$response = wp_remote_get( $url, $args );

		try {
			return $this->handle_response( $response );
		} catch ( WeeConnectPayException $e ) {
			if ( is_admin() /*$e->shouldDisplayAdminNoticeError()*/ ) {
				return new WP_Error( 'wc-order', $e->getMessage()/*__( 'Order has been already refunded', 'weeconnectpay' )*/ );
			}
			return false;
		} catch ( \Exception $e ){
			// In case an exception other than one of ours happen,
			// we still want to end the refund attempt properly
			return false;
		}
	}

	private function api_post( $url, $data ) {
		$args     = $this->set_request_args( $data );
		$response = wp_remote_post( $url, $args );

		try {
			return $this->handle_response( $response );
		} catch ( WeeConnectPayException $e ) {
			if ( is_admin() /*$e->shouldDisplayAdminNoticeError()*/ ) {

				return new WP_Error( 'wc-order', $e->getMessage()/*__( 'Order has been already refunded', 'weeconnectpay' )*/ );
			}

			return false;
		}
	}

	/**
	 * @param $data
	 *
	 * @return array
	 */
	private function set_request_args( $data = null ) {

		return array(
			'headers' => array(
				'Accept'        => 'application/json',
				'Content-Type'  => 'application/json',
				'Authorization' => "Bearer {$this->integrationSettings->getAccessToken()->getToken()}",
				'Integration-Uuid' => $this->integrationSettings->getIntegrationUuid(),
			),
			'body'    => $data,
			'timeout' => 15,
		);
	}

	/**
	 * @param $response
	 *
	 * @return bool|mixed Returns an actual response or false if an error occurs.
	 * @throws WeeConnectPayException
	 */
	private function handle_response( $response ) {
		if ( is_wp_error( $response ) ) {
			if ( $this->is_debug ) {
			}

			return false;
		}
		$response_content = json_decode( wp_remote_retrieve_body( $response ) );
		$http_code        = wp_remote_retrieve_response_code( $response );
		if ( is_array( $response ) ) {
			if ( 200 === $http_code || 201 === $http_code ) {
				return $response_content;
			} else {
				if ( isset( $response_content->error, $response_content->error->message ) ) {
					throw new WeeConnectPayException( $response_content->error->message);
				} else {
					return false;
				}
			}
		}
		return false;
	}

	public function create_order( $clover_order ) {

		$endpoint = $this->url_api . '/v1/woocommerce/orders';
		$body     = $clover_order;

		$response = wp_remote_post(
			$endpoint,
			array(
				'method'      => 'POST',
				'timeout'     => 60,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => true,
				'headers'     => array(
					'Content-Type'  => 'application/json',
					'Accept'        => 'application/json',
					'Authorization' => "Bearer {$this->integrationSettings->getAccessToken()->getToken()}",
					'Integration-Uuid' => $this->integrationSettings->getIntegrationUuid(),

				),
				'body'        => wp_json_encode( $body ),
				'cookies'     => array(),
			)
		);

		// Response Body
		$body = wp_remote_retrieve_body( $response );

		// On success (i.e. `$body` is a valid JSON string), `$response_data` would be an array
		return ( ! is_wp_error( $response ) ) ? json_decode( $body, true ) : null;

	}

	public function refund_woocommerce_order( $refund_payload ) {

		$endpoint = $this->url_api . '/v1/woocommerce/orders/refund';

		$response = $this->api_post( $endpoint, json_encode( $refund_payload ) );

		return $response;
	}

	public function set_integration_name_from_option_name( $integration_option_name ): string {
		if ( 'woocommerce_weeconnectpay_settings' === $integration_option_name ) {
			return 'woocommerce';
		} else {
			return 'unknown';
		}
	}

	/**
	 * @param WC_Order $order
	 *
	 * @return mixed|null
	 * @throws MissingStateException
	 * @deprecated since 2.0
	 */
	public function prepare_order_for_payment( WC_Order $order ) {
		$payment_processing_source      = 'clover';
		$payment_processing_source_type = 'ecom';

			if ( 'clover' === $payment_processing_source && 'ecom' === $payment_processing_source_type ) {

				// If taxes are included -- To help with future order manipulations
				$order->add_meta_data( 'weeconnectpay_tax_included', true );
				$order->add_meta_data( 'weeconnectpay_merged_qty', true );
				// To be able to properly refund shipping as line items if it is and keep a normal refund flow otherwise.
				$order->add_meta_data( 'weeconnectpay_shipping_as_clover_line_item', true );
				// To be able to fetch the name regardless of localization -- for refunds etc
				$order->add_meta_data( 'weeconnectpay_shipping_line_item_name', 'Shipping Fees' );
				$order->save_meta_data();

				$amount_in_cents = WeeConnectPayHelper::safe_amount_to_cents_int( $order->get_total() );

				$woocommerce_internal_order = $this->prepare_internal_woocommerce_order( $order, $payment_processing_source, $payment_processing_source_type );

				$clover_order = $this->prepare_clover_order( $order );

				$is_matching_amounts = $this->is_matching_amounts( $clover_order, $amount_in_cents );
				if ( $is_matching_amounts ) {
					$order_creation_response = $this->create_order(
						array(
							'pp_order'               => $clover_order,
							'woocommerce_order_info' => $woocommerce_internal_order,
							'pp_order_id'            => $order->get_meta( 'weeconnectpay_clover_order_uuid' ),
						)
					);

					if ( ! isset( $order_creation_response['id'] ) ) {
						return null;
					} else {
						return $order_creation_response;
					}
				} else {
					return null;
				}
			} else {
				// woocommerce but not clover ecom
				return null;
			}
	}

	public function get_expanded_merchant_inventory() {

		$endpoint = $this->url_api . '/v1/clover/merchants/current/expanded_items';

		return $this->api_get( $endpoint );
	}

	/**
	 * Retrieves the API domain based on the environment
	 *
	 * @since 1.4.0
	 * @return string
	 */
	public static function getApiDomain(): string {
		$wp_env = WeeConnectPayUtilities::get_wp_env();

		switch ( $wp_env ) {
			case 'local':
			case 'development':
				return self::LOCAL_API_URL;
			case 'staging':
				return self::STAGING_API_URL;
			case 'production':
			default:
				return self::PRODUCTION_API_URL;
		}
	}
}
