<?php

namespace WeeConnectPay\WordPress\Plugin\includes;

use WeeConnectPay\CloverApp;
use WeeConnectPay\CloverMerchant;
use WeeConnectPay\Exceptions\SettingsInitializationException;
use WeeConnectPay\Exceptions\WeeConnectPayException;
use WeeConnectPay\Integrations\IntegrationSettings;
use WeeConnectPay\Settings;
use WeeConnectPay\StandardizedResponse;
use WP_Error;
use WP_REST_Controller;
use WP_REST_Response;

class WeeConnectPayController extends WP_REST_Controller {

	protected $namespace = 'weeconnectpay/v1';
	private $wp_env;
	private $url_api;


	public function __construct() {

		$this->wp_env = WeeConnectPayUtilities::get_wp_env();

		switch ( $this->wp_env ) {
			case 'local':
			case 'development':
				// Do dev stuff
				$this->url_api = 'https://weeconnect-api.test';
				break;
			case 'staging':
				// Do staging stuff
				$this->url_api = 'https://apidev.weeconnectpay.com';
				break;
			case 'production':
			default:
				// Do production stuff
				$this->url_api = 'https://api.weeconnectpay.com';
		}
	}

	public function register_routes() {
		// Plugin Registration request from our API
		$this->registerIntegrationRegistrationRoute();

		// Plugin dependency check
		$this->registerPluginDependencyCheckRoute();
		$this->getPluginSettings();
	}

	/**
	 * Not currently used
	 */
	public function registerPluginDependencyCheckRoute() {
		$namespace = 'weeconnectpay/v1';
		$path      = 'integration/dependencies';

		register_rest_route(
			$namespace,
			'/' . $path,
			array(
				array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_integration_dependencies' ),
					'permission_callback' => array( $this, 'get_integration_dependencies_permissions_check' ),
				),
			)
		);
	}

	/**
	 * Registers the plugin settings route
	 *
	 * @return void
	 */
	public function getPluginSettings() {
		$namespace = 'weeconnectpay/v1';
		$path      = 'plugin/settings';

		register_rest_route(
			$namespace,
			'/' . $path,
			array(
				array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_plugin_settings' ),
					'permission_callback' => array( $this, 'get_plugin_settings_permissions_check' ),
				),
			)
		);
	}

	public function get_plugin_settings_permissions_check( $request ): bool {
		return true;
	}

	/**
	 * @return object StandardizedResponse::emitError | StandardizedResponse::emitSuccess
	 */
	public function get_plugin_settings() {
		try {
			$integrationSettings = new IntegrationSettings();
			// TODO: Make a settings response object

			// Should either return the login page settings OR the full settings we need to display.
			$merchant = CloverMerchant::getFromIntegrationSettings( $integrationSettings);
			$app = CloverApp::getFromIntegrationSettings( $integrationSettings);
			return StandardizedResponse::emitSuccess( json_decode(json_encode( ['app' => $app, 'merchant' => $merchant])));
		} catch ( WeeConnectPayException $e ) {
			return StandardizedResponse::emitError( $e->toObject() );
		}
	}


	public function get_integration_dependencies_permissions_check( $request ): bool {
		return true;
	}

	/**
	 * @deprecated
	 * @return object StandardizedResponse::emitError | StandardizedResponse::emitSuccess
	 */
	public function get_integration_dependencies(): object {
		try {
			$integration_settings = new IntegrationSettings();
			//$integration_settings->retrieve();
			return $integration_settings->validateDependencies();
		} catch ( WeeConnectPayException $e ) {
			return StandardizedResponse::emitError( $e->toObject() );
		}
	}

	public function set_plugin_configs_permissions_check( $request ): bool {
		return true;
	}

	public function set_plugin_configs( $request ) {

		$body = json_decode( $request->get_body() );

		// get values needed
		if ( ! isset( $body->integration_id, $body->auth_hash ) ) {
			return new WP_Error( 'invalid_request', 'the request is missing required parameters', array( 'status' => 400 ) );
		}

		$response = wp_remote_post(
			$this->url_api . "/v1/integration/$body->integration_id/token",
			array(
				'headers' => array(
					'Accept'       => 'application/json',
					'Content-Type' => 'application/json',
				),
				'body'    => wp_json_encode( array( 'auth_hash' => $body->auth_hash ) ),
				'timeout' => 15,
			)
		);

		$payload = json_decode( wp_remote_retrieve_body( $response ) ); // stringified json sent from our API, decode it.

		// Check payload
		if ( ! isset(
			$payload->integration->uuid,
			$payload->integration->token,
			$payload->integration->clover_merchant->name,
			$payload->integration->clover_merchant->uuid,
			$payload->integration->clover_app->name,
			$payload->integration->clover_app->uuid,
			$payload->integration->clover_employee->name,
			$payload->integration->clover_employee->uuid,
			$payload->integration->integrable_backend->db_prefix
		) ) {
			return new WP_Error( 'invalid_response', 'the response is missing required attributes for us to continue', array( 'status' => 400 ) );
		}

		$integration_settings = new IntegrationSettings();
		try {
			$integration_settings->saveAfterAuth( $payload );

			return StandardizedResponse::emitSuccess( (object) ['message' => 'Settings were properly saved after authenticating.']);
		} catch ( SettingsInitializationException $exception ) {

			return StandardizedResponse::emit( $exception->toObject() );
		} catch ( \Throwable $exception ) {

			return StandardizedResponse::emit( $exception );
		}

		//		return new WP_REST_Response( $is_success, $is_success? 200 : 500 );
	}

	private function registerIntegrationRegistrationRoute(): void {
		$namespace = 'weeconnectpay/v1';
		$path      = 'plugin/register';

		register_rest_route(
			$namespace,
			'/' . $path,
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'set_plugin_configs' ),
					'permission_callback' => array( $this, 'set_plugin_configs_permissions_check' ),
				),

			)
		);
	}

}
