<?php
// phpcs:disable WordPress.NamingConventions.ValidVariableName.PropertyNotSnakeCase
// phpcs:disable WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase
// phpcs:disable WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
// phpcs:disable WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase


namespace WeeConnectPay\WordPress\Plugin\includes;

/**
 * Class WeeConnectPayUtilities
 * holds static utility functions to parse and return modified results
 *
 * @package WeeConnectPay\WordPress\Plugin\includes
 */
class WeeConnectPayUtilities {

	/**
	 *
	 * @param string $string snake_case string to be turned into camelCase
	 * @param bool $returnPascalCase Option to return CamelCase instead of pascalCase
	 *
	 * @return string|string[]
	 */
	public static function dashesToCamelCase( string $string, $returnPascalCase = false ) {
		$str = str_replace( '_', '', ucwords( $string, '_' ) );

		if ( ! $returnPascalCase ) {
			$str = lcfirst( $str );
		}

		return $str;
	}

	/**
	 * @param $orderId
	 *
	 * @return string
	 * @since 1.0.0
	 */
	public static function getWooCommerceOrderInstructions( $orderId ): string {
		$orderId = intval( $orderId );
		$post    = get_post( $orderId );

		if ( ! $post ) {
			return '';
		}

		return $post->post_excerpt;
	}


	/**
	 * Checks if wp_get_environment_type exists first. If not, returns production, otherwise returns the WP_ENVIRONMENT_TYPE constant. ( local/development, staging or production ( default ) only )
	 * @return string
	 */
	public static function get_wp_env(): string {

		$wp_env = null;

		if ( function_exists( 'wp_get_environment_type' ) ) {
			$wp_env = wp_get_environment_type();
		}

		switch ( $wp_env ) {
			case 'local':
			case 'development':
				return 'development';

			case 'staging':
				return 'staging';

			case 'production':
			default:
				return 'production';
		}
	}

	/**
	 * Get WeeConnectPay settings URL
	 */
	public static function getSettingsURL() {
		return admin_url() . 'admin.php?page=wc-settings&tab=checkout&section=weeconnectpay';
	}

	/**
	 * Converts a version string ( major, minor, patch -- Everything else will be excluded ) into an unsafe array of numbers.
	 *
	 * @param string $version
	 *
	 * @return array
	 * @example "5.9-alpha-52003" ---> [5,9]
	 * @since 1.3.9
	 */
	public static function stringVersionToArray( string $version ): array {
		return explode( '.', $version, 3 );
	}

	/**
	 * Cleans the version segments to avoid issues when inserting versions like "5.9-alpha-52003"
	 *
	 * @param array $version_array
	 *
	 * @return array
	 * @since 1.3.9
	 */
	public static function cleanVersionArray( array $version_array ): array {
		$clean_array = [];
		foreach ( $version_array as $version_segment ) {
			$clean_version_segment = preg_replace( '/\D.*/', '', $version_segment );
			$clean_array[]         = $clean_version_segment;
		}

		return $clean_array;
	}

	/**
	 * Converts a version string ( major, minor, patch -- Everything else will be excluded and replaced with 0 if missing ) into a safe array of numbers.
	 *
	 * @param string $version
	 *
	 * @return array
	 * @example "5.9-alpha-52003" ---> [5,9,0]
	 * @since 1.3.9
	 */
	public static function stringVersionToCleanArray( string $version ): array {
		$dirty_version_array = self::stringVersionToArray( $version );
		$version_array       = self::cleanVersionArray( $dirty_version_array );

		// Fill in the blanks for any missing version parts before returning
		return [
			$version_array[0] ?? 0,
			$version_array[1] ?? 0,
			$version_array[2] ?? 0
		];
	}

	/**
	 * Takes the current user locale, or the current WordPress locale if missing
	 * and returns one of the supported SDK locale
	 *
	 * @return string
	 * @since 2.0.0
	 */
	public static function getLocale(): string {
		$wp_locale = get_user_locale();
		switch ($wp_locale) {
			case 'en_CA':
				return 'en-CA';
			case 'fr_FR': // Fallback to the only supported FR -- They also have postal codes
			case 'fr_CA':
				return 'fr-CA';
			default:
				return 'en';
		}
	}

}
