<?php


namespace WeeConnectPay\Integrations;

use WeeConnectPay\Dependencies\GuzzleHttp\Exception\ClientException;
use WeeConnectPay\Dependencies\GuzzleHttp\Exception\GuzzleException;
use WC_Admin_Settings;
use WeeConnectPay\Api\ApiEndpoints;
use WeeConnectPay\Api\Requests\VerifyAuthenticationRequest;
use WeeConnectPay\Exceptions\Codes\ExceptionCode;
use WeeConnectPay\Exceptions\IntegrationPermissionsException;
use WeeConnectPay\Exceptions\SettingsInitializationException;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayAPI;
use WeeConnectPay\WordPress\Plugin\includes\WeeConnectPayUtilities;
use WP_Error;

class Authentication {

	/**
	 * Helps us create the handle for authentication callback
	 * @throws \WeeConnectPay\Exceptions\WeeConnectPayException
	 * @since 1.4.0
	 *
	 */
	public static function fetchIntegrationId() {

		if ( !current_user_can( 'manage_options') ) {
			return new WP_Error( 'unauthorized', 'Admin access is needed to prepare authentication.', array( 'status' => 403 ) );
		}

		$wp_env = WeeConnectPayUtilities::get_wp_env();

		switch ( $wp_env ) {
			case 'local':
			case 'development':
				// Do dev stuff
				$url_api = 'https://weeconnect-api.test';
				break;
			case 'staging':
				// Do staging stuff
				$url_api = 'https://apidev.weeconnectpay.com';
				break;
			case 'production':
			default:
				// Do production stuff
				$url_api = 'https://api.weeconnectpay.com';
		}

		global $wpdb, $wp_version;

		$domain = sanitize_url( $_SERVER['SERVER_NAME']);
		$woocommerce_settings_url = WeeConnectPayUtilities::getSettingsURL();

		// Endpoint to create or update the integration and get the UUID back
		$endpoint = $url_api . '/v1/integrations/woocommerce';

		$body =
			array(
				'domain'             => $domain,
				'integrable'         => array(
					'type'    => 'woocommerce',
					'version' => defined( 'WC_VERSION' ) ? WC_VERSION : null,
				),
				'integrable_backend' => array(
					'type'         => 'wordpress',
					'home_url'     => home_url(),
					'site_url'     => site_url(),
					'settings_url' => $woocommerce_settings_url,
					'plugins_url'  => plugins_url(),
					'db_prefix'    => $wpdb->prefix,
					'version'      => $wp_version,
				),
			);


		$response = wp_remote_post(
			$endpoint,
			array(
				'method'      => 'POST',
				'timeout'     => 60,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => true,
				'headers'     => array(
					'Content-Type' => 'application/json',
					'Accept'       => 'application/json',
				),
				'body'        => wp_json_encode( $body ),
				'cookies'     => array(),
			)
		);

		// Response Body
		$body                 = wp_remote_retrieve_body( $response );
		$response_body_object = json_decode( $body );

		if ( ! isset( $response_body_object->integration_id ) ) {
			return new WP_Error( 'invalid_response', 'There is no integration handle in the popup authentication preparation response.', array( 'status' => 400 ) );
		}

		return $response_body_object->integration_id;
	}

	/**
	 * Verify that the integration is authenticated properly.
	 *
	 * @param string $integrationUuid
	 *
	 * @return bool
	 */
	public static function isValid( string $integrationUuid ): bool {

		try {
			( new VerifyAuthenticationRequest() )->GET( $integrationUuid );

			return true;
		} catch ( ClientException $exception ) {
			return false;
		}
	}
}
