<?php
/** @noinspection PhpIncludeInspection */

/* phpcs:disable WordPress
 * phpcs:disable Generic.Arrays.DisallowShortArraySyntax */


namespace WeeConnectPay\Integrations;

require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Api/ApiClient.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Api/ApiEndpoints.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Api/Requests/VerifyAuthenticationRequest.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Api/Requests/CreateCloverOrderChargeRequest.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'StandardizedResponse.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/Codes/ExceptionCode.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/WeeConnectPayException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/SettingsInitializationException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/MissingDependencyException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/MissingStateException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/InsufficientDependencyVersionException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Exceptions/IntegrationPermissionsException.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'AccessToken.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'CloverApp.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'CloverCountry.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'CloverEmployee.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'CloverMerchant.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'CloverMerchantAppSubscription.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Currency.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Dependency.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Validators/DependencyValidator.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Settings.php';
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'Integration/Authentication.php';


use WeeConnectPay\{AccessToken,
	CloverApp,
	CloverCountry,
	CloverEmployee,
	CloverMerchant,
	CloverMerchantAppSubscription,
	Currency,
	Dependency,
	Exceptions\Codes\ExceptionCode,
	Exceptions\InsufficientDependencyVersionException,
	Exceptions\MissingDependencyException,
	Exceptions\SettingsInitializationException,
	Exceptions\WeeConnectPayException,
	Settings,
	StandardizedResponse,
	WordPress\Plugin\includes\WeeConnectPayAPI,
	WordPress\Plugin\includes\WeeConnectPayUtilities};

class IntegrationSettings extends Settings implements \JsonSerializable {


	public const PLUGIN_OPTION_PREFIX = 'weeconnectpay_';
	public const DB_KEY_WOOCOMMERCE_INTEGRATION = 'woocommerce_weeconnectpay_settings';
	public const DB_KEY_SUFFIX_INTEGRATION_UUID = 'integration_uuid';
	public const DB_KEY_SUFFIX_PLATFORM_TYPE = 'platform_type';

	public const INTEGRATION_NAME = Dependency::WORDPRESS;

	/**
	 * External dependencies with their minimum version for the integration to run properly
	 *
	 * @since 1.3.9
	 * @updated 1.4.3
	 */
	public const DEPENDENCIES = [
		Dependency::PHP         => [ 7, 2, 0 ],
		Dependency::WORDPRESS   => [ 5, 4, 0 ],
		Dependency::WOOCOMMERCE => [ 3, 0, 4 ]
	];


	/**
	 * @var bool $isWoocommerceGatewayEnabled
	 */
	public $isWoocommerceGatewayEnabled = false; // WP Specific, WooCommerce Specific

	/**
	 * @var string
	 */
	public $woocommerceGatewayTitle = '';


	/**
	 * @var string $wpdbPrefix
	 */
	public $wpdbPrefix; // WP specific

	/**
	 * @var string $integrationUuid
	 */
	public $integrationUuid;

	/**
	 * @var string $platformType
	 */
	public $platformType;

	/**
	 * @return string
	 */
	public function getPlatformType(): string {
		if ( ! $this->platformType ) {
			$this->platformType = get_option( self::PLUGIN_OPTION_PREFIX . self::DB_KEY_SUFFIX_PLATFORM_TYPE );
		}

		return $this->platformType;
	}

	/**
	 * @param string $platformType
	 *
	 * @return void
	 * @throws WeeConnectPayException
	 */
	public function setPlatformType( string $platformType ): void {
		$this->createOrUpdateSetting( self::DB_KEY_SUFFIX_PLATFORM_TYPE, $platformType, $this->platformType );
	}


	/**
	 * @return bool
	 */
	public function isWoocommerceGatewayEnabled(): bool {

		if ( ! $this->isWoocommerceGatewayEnabled ) {
			$wc_wcp_options = get_option( self::DB_KEY_WOOCOMMERCE_INTEGRATION );

			if ( $wc_wcp_options && isset( $wc_wcp_options->enabled ) ) {
				$this->isWoocommerceGatewayEnabled = $wc_wcp_options->enabled;
			}
		}

		return $this->isWoocommerceGatewayEnabled;
	}

	/**
	 * @param bool $isWoocommerceGatewayEnabled
	 *
	 * @return void
	 * @throws WeeConnectPayException
	 */
	public function setIsWoocommerceGatewayEnabled( bool $isWoocommerceGatewayEnabled ): void {

		$optionString = self::DB_KEY_WOOCOMMERCE_INTEGRATION;
		$oldOption    = get_option( $optionString );

		// Has WooCommerce saved our gateway's options?
		if ( ! $oldOption ) {
			throw new WeeConnectPayException( __( 'Failed to update the WooCommerce WeeConnectPay gateway enabled status in the database. The gateway options could not be found.', 'weeconnectpay' ), ExceptionCode::SETTINGS_UPDATE_EXCEPTION );
		}

		// Is the specific option in the DB the same as the one we're trying to update?
		if ( $oldOption->enabled === $isWoocommerceGatewayEnabled ) {
			$oldOption->enabled = $isWoocommerceGatewayEnabled;
		} else {
			$updated = update_option( $optionString, $oldOption );

			// Only update the object if we successfully updated in the DB
			if ( $updated === true ) {
				$this->isWoocommerceGatewayEnabled = $isWoocommerceGatewayEnabled;
			} else {
				throw new WeeConnectPayException( __( 'Failed to update the WooCommerce integration status in the database.', 'weeconnectpay' ), ExceptionCode::SETTINGS_UPDATE_EXCEPTION );
			}
		}
	}


	/**
	 * @return string
	 */
	public function getWoocommerceGatewayTitle(): string {

		if ( ! $this->woocommerceGatewayTitle ) {
			$wc_wcp_options = get_option( self::DB_KEY_WOOCOMMERCE_INTEGRATION );

			if ( $wc_wcp_options && isset( $wc_wcp_options->title ) ) {
				$this->woocommerceGatewayTitle = $wc_wcp_options->title;
			}
		}

		return $this->woocommerceGatewayTitle;
	}

	/**
	 * @param string $woocommerceGatewayTitle
	 *
	 * @return void
	 * @throws WeeConnectPayException
	 */
	public function setWoocommerceGatewayTitle( string $woocommerceGatewayTitle ): void {
		$optionString = self::DB_KEY_WOOCOMMERCE_INTEGRATION;
		$oldOption    = get_option( $optionString );

		// Has WooCommerce saved our gateway's options?
		if ( ! $oldOption ) {
			throw new WeeConnectPayException( __( 'Failed to update the WooCommerce integration title in the database. The gateway options could not be found.', 'weeconnectpay' ), ExceptionCode::SETTINGS_UPDATE_EXCEPTION );
		}

		// Is the specific option in the DB the same as the one we're trying to update?
		if ( $oldOption->title === $woocommerceGatewayTitle ) {
			$this->woocommerceGatewayTitle = $woocommerceGatewayTitle;
		} else {
			$updated = update_option( $optionString, $oldOption );

			// Only update the object if we successfully updated in the DB
			if ( $updated === true ) {
				$this->woocommerceGatewayTitle = $woocommerceGatewayTitle;
			} else {
				throw new WeeConnectPayException( __( 'Failed to update the WooCommerce integration title in the database.', 'weeconnectpay' ), ExceptionCode::SETTINGS_UPDATE_EXCEPTION );
			}
		}
	}


	/**
	 * Retrieves the WeeConnectPay integration ID from the integration instance, and tries to set it from the integration DB if it is unset.
	 *
	 * @return string
	 * @throws SettingsInitializationException
	 */
	public function getIntegrationUuid(): string {
		if ( ! $this->integrationUuid ) {
			$this->integrationUuid = get_option( self::PLUGIN_OPTION_PREFIX . self::DB_KEY_SUFFIX_INTEGRATION_UUID );
		}

		if ( gettype($this->integrationUuid) === 'string' ) {
			return $this->integrationUuid;
		} else {
			throw new SettingsInitializationException( "Could not retrieve a valid WeeConnectPay integration ID from the database.", ExceptionCode::SETTINGS_RETRIEVAL_EXCEPTION );
		}
	}

	/**
	 * @param string $integrationUuid
	 *
	 * @return void
	 * @throws WeeConnectPayException
	 */
	private function setIntegrationUuid( string $integrationUuid ): void {
		$this->createOrUpdateSetting( self::DB_KEY_SUFFIX_INTEGRATION_UUID, $integrationUuid, $this->integrationUuid );
	}


	/**
	 * @deprecated
	 * @return object StandardizedResponse::emitError | StandardizedResponse::emitSuccess
	 */
	public function validateDependencies(): object {
		try {
			foreach ( self::DEPENDENCIES as $dependencyName => $minVerArray ) {
				$dependency = new Dependency( $dependencyName, $minVerArray );

				if ( $dependency->validate() !== true ) {
					throw new WeeConnectPayException(
						sprintf(
							__( 'Something went wrong while validating the current %1$s version. %1$s is required for this integration to work properly. Integration disabled. ', 'weeconnectpay' ),
							$dependencyName
						),
						ExceptionCode::UNHANDLED_EXCEPTION
					);
				}
			}

			return StandardizedResponse::emitSuccess( (object) [ 'message' => __( 'Dependencies are ok!', 'weeconnectpay' ) ] );
		} catch ( MissingDependencyException | InsufficientDependencyVersionException | WeeConnectPayException $exception ) {
			return StandardizedResponse::emitError( $exception->toObject() );
		} catch ( \Throwable $exception ) {

			return StandardizedResponse::emitError(
				( new WeeConnectPayException(
					__( 'Something went wrong while validating one of the requirements for this integration to work properly. Integration disabled.', 'weeconnectpay' ),
					ExceptionCode::UNHANDLED_EXCEPTION
				) )->toObject()
			);
		}

	}

	/**
	 * Attempts to retrieve every setting needed for payment processing. Returns true if successful.
	 *
	 * @since 1.3.9
	 * @return bool
	 */
	public function arePaymentProcessingSettingsReady(): bool {
		try {
			// app sub
			$this->getSubscription();
			// clover app
			$this->getCloverApp();
			// clover emp
			$this->getCloverEmployee();
			// clover merchant
			$this->getCloverMerchant();
			// clover pakms
			$this->getPublicAccessKey();
			// integration uuid
			$this->getIntegrationUuid();
			// jwt
			$this->getAccessToken();
			// platform type
			$this->getPlatformType();

			return true;
		} catch ( \Throwable $exception ) {
			return false;
		}
	}

//	// Should either return the login page settings OR the full settings we need to display.
//	public function prepareAppSettingsForApi() {
//		try {
//
//		}
//	}


	/**
	 *  To be used during plugin activation.
	 */
	public static function maybeFirstTimeInit() {
		if ( ! ( new self )->arePaymentProcessingSettingsReady() ) {

			try {
				$new_install_settings = new IntegrationSettings();
				$integration_id = Authentication::fetchIntegrationId();
				$new_install_settings->setIntegrationUuid( $integration_id );
			} catch ( \Throwable $exception ) {
			}
		}
	}

	/**
	 * @param object $object
	 *
	 * @return void
	 * @throws WeeConnectPayException
	 */
	public function saveAfterAuth( object $object ): void {
			$this->setIntegrationUuid( $object->integration->uuid );
			$this->setPlatformType( $object->integration->integrable_type );
			//$this->setWpdbPrefix( $object->integration->hosted_platform->hostable->db_prefix );
			$this->setCloverMerchant(
				new CloverMerchant(
					$object->integration->clover_merchant->name,
					$object->integration->clover_merchant->uuid
				)
			);
			$this->setCloverEmployee(
				new CloverEmployee(
					$object->integration->clover_employee->name,
					$object->integration->clover_employee->uuid
				)
			);
			$this->setCloverApp(
				new CloverApp(
					$object->integration->clover_app->name,
					$object->integration->clover_app->uuid
				)
			);
			$this->setAccessToken(
				new AccessToken(
					$object->integration->token->accessToken,
					$object->integration->token->token->scopes,
					$object->integration->token->token->name,
					$object->integration->token->token->expires_at
				)
			);
			$this->setPublicAccessKey( $object->integration->clover_access_token->public_key->key );

			$acceptedCurrencies = Currency::CreateMany(
				$object->integration->clover_access_token->app_subscription->country->currencies
			);

			$country = new CloverCountry(
				$object->integration->clover_access_token->app_subscription->country->name,
				$object->integration->clover_access_token->app_subscription->country->alpha2_code,
				$object->integration->clover_access_token->app_subscription->country->alpha3_code
			);

			$this->setSubscription(
				new CloverMerchantAppSubscription(
					$object->integration->clover_access_token->app_subscription->uuid,
					$object->integration->clover_access_token->app_subscription->name,
					$country,
					$acceptedCurrencies
				)
			);
	}


	/**
	 * @inheritDoc
	 * @return array a JSON representation of this class
	 */
	public function jsonSerialize(): array {
		$objectArray = [];
		foreach ( $this as $key => $value ) {
			$objectArray[ $key ] = $value;
		}

		return $objectArray;
	}


	/**
	 * Get the redirect URL for the authentication redirect using the integration ID
	 *
	 * @since 1.4.0
	 * @return string
	 * @throws SettingsInitializationException
	 */
	public static function redirectUrl(): string {
		return WeeConnectPayAPI::getApiDomain() . '/login/clover?intent=authorize-redirect&integration_id=' . ( new IntegrationSettings )->getIntegrationUuid();
	}


	/**
	 * Checks if the authentication settings are valid
	 *
	 * @since 1.4.0
	 * @return bool
	 */
	public function isAuthValid(): bool {
		try {
			// Will throw an exception if missing -- No ID = Not logged in properly
			$integration_id = $this->getIntegrationUuid();

			if ( $this->accessTokenExists() ) {
				return Authentication::isValid( $integration_id );
			} else {
				return false;
			}

		} catch ( WeeConnectPayException $exception ) {
			return false;
		}
	}
}
