<?php

/* phpcs:disable WordPress
 * phpcs:disable Generic.Arrays.DisallowShortArraySyntax */
namespace WeeConnectPay\Validators;

use WeeConnectPay\Dependency;
use WeeConnectPay\Exceptions\InsufficientDependencyVersionException;
use WeeConnectPay\Exceptions\MissingDependencyException;
use WeeConnectPay\Exceptions\Codes\ExceptionCode;

class DependencyValidator {


	/**
	 * @var Dependency
	 */
	private $dependency;

	/**
	 * DependencyValidator constructor.
	 *
	 * @param Dependency $dependency
	 */
	public function __construct( Dependency $dependency ) {
		$this->dependency = $dependency;
	}


	/**
	 * @param array $versionToValidate
	 *
	 * @return bool
	 * @throws InsufficientDependencyVersionException
	 * @throws MissingDependencyException
	 */
	public function validate( array $versionToValidate ): bool {
		if ( $versionToValidate === null || count( $versionToValidate ) <= 0 ) {
			throw new MissingDependencyException(
				sprintf(
					__( '%1$s does not seem to be active, which is required for this integration to work properly. Integration disabled.', 'weeconnectpay' ),
					$this->dependency->name
				),
				ExceptionCode::MISSING_DEPENDENCY
			);
		}

		$version_to_validate_string = $this->versionArrayToString( $versionToValidate );
		$dependency_version_string  = $this->versionArrayToString( $this->dependency->minVer );

		if ( ! $this->versionIsSufficient( $version_to_validate_string, $dependency_version_string ) ) {
			throw new InsufficientDependencyVersionException(
				sprintf(
					__( '%1$s version %2$s or higher is required for this integration to work. Your current version is %3$s. Integration disabled.' , 'weeconnectpay'),
					$this->dependency->name,
					$dependency_version_string,
					$version_to_validate_string
				),
				ExceptionCode::DEPENDENCY_VERSION_INSUFFICIENT
			);
		} else {
			return true;
		}
	}

	public function versionIsSufficient( string $actualVer, string $dependencyVer ): bool {
		return version_compare( $actualVer, $dependencyVer, '>=' );
	}

	/**
	 * @param array $versionArray
	 *
	 * @return string
	 */
	public function versionArrayToString( array $versionArray ): string {
		return implode( '.', $versionArray );
	}
}
